#ifndef GLOBALVARIABLE_H
#define GLOBALVARIABLE_H

#include <QString>
#include <vector>
#include <list>
#include <map>
#include <mutex>
#include <QDateTime>

#include "nlohmann/json.hpp"


class FuncBase;

/* ====================================================================================
 * *******************************    全局变量定义    **********************************
 * ====================================================================================*/

/**
 * @brief 全局信息
 * 
 */

using nJson = nlohmann::json;

/* Rides数据时间有效性 */
extern int g_eventTimeVaild;




/* 线程运行时的状态 */
enum class RunTimeState
{
    RUN_STATE_NONE = 0,         /* 无状态 */
    RUN_STATE_INIT,             /* 初始化 */
    RUN_STATE_RUN,              /* 运行 */
    RUN_STATE_STOP,             /* 停止，是外部给子线程的信号，让其退出 */
    RUN_STATE_ERROR,            /* 错误 */
    RUN_STATE_EXITCOMPLET,      /* 退出完成 */
};

/* 应用功能 */
enum class AppFunction
{
    APP_NONE = 0,               /* 无功能 */
    APP_OnWork = 1,             /* 人员在岗识别（需要多个摄像机配合） */
    APP_Contraband,             /* 违禁品识别 */
    APP_Illegal,                /* 非法入侵检测（需要多个摄像机配合） */
    APP_Fatigue,                /* 疲劳检测 */
    APP_Regional,               /* 区域人员检测，人员计数（需要多个摄像机配合） */
    APP_Mouse,                  /* 老鼠识别 */
    APP_PlayPhone,              /* 玩手机识别 */
    APP_NoMask,                 /* 未戴口罩识别 */
    APP_AllDown,                /* 摔倒识别 */
};

/**
 * @brief 房间类型
 * 
 */
enum class Enum_RoomType
{
    ROOM_NONE = 0,              /* 无房间 */
    ROOM_LIVE = 1,              /* 直播间 */
    ROOM_DIC = 2,               /* 导播间 */
    ROOM_LIVEDIC = 3,           /* 直播间和导播间 */
};


/**
 * @brief 时段类型
 * 
 */
enum class Enum_PeriodType
{
    PERIOD_ALL = 0,         /* 所有时段 */
    PERIOD_DAY,             /* 按天 */
    PERIOD_WEEK,            /* 按周 */
};
 


/* 算法相关信息 */
struct AlgorithmInfo
{
    int PKID;                   /* 主键ID */
    int ActionTaskID;           /* 算法任务ID */
    std::string ActionID;       /* 算法ID */
    std::string ActionName;     /* 算法名称 */
    

    AlgorithmInfo() = default;
    AlgorithmInfo(const AlgorithmInfo& other)
        : PKID(other.PKID), ActionID(other.ActionID), ActionName(other.ActionName), ActionTaskID(other.ActionTaskID) {}
    AlgorithmInfo& operator=(const AlgorithmInfo& other) {
        if (this != &other) {
            ActionID = other.ActionID;
            ActionName = other.ActionName;
            ActionTaskID = other.ActionTaskID;
            PKID = other.PKID;
        }
        return *this;
    }
    /* 对比是否相等 */
    bool operator==(const AlgorithmInfo& other) const {
        if(ActionID != other.ActionID) {
            return false;
        }
        if(ActionName != other.ActionName || ActionTaskID != other.ActionTaskID) {
            return false;
        }
        return true;
    }
};


/* 设备列表 */
struct DeviceInfo
{
    int PKID;                   /* 主键ID */
    int DeviceID;               /* 设备ID */
    int DevicePort;             /* 设备端口 */
    std::string DeviceName;     /* 设备名称 */
    std::string DeviceIP;       /* 设备IP */
    std::string DeviceSerial;   /* 设备序列号，这里好像是IP */
    std::string DeviceType;     /* 设备类型 */
    std::string UserAccount;    /* 用户账号 */
    std::string UserPassword;   /* 用户密码 */
    std::vector<AlgorithmInfo> vecAlgorithmInfo; /* 算法信息 */

    DeviceInfo() = default;
    DeviceInfo(const DeviceInfo& other);
    DeviceInfo& operator=(const DeviceInfo& other);
    /* 对比是否相等 */
    bool operator==(const DeviceInfo& other);

    /* 对比设备关联的算法信息是否相等 */
    bool isEqualAlgorithmInfo(const std::vector<AlgorithmInfo>& other);
};


/* 算法和设备关联的相关信息 */
struct DeviceAlgorithmInfo
{
    int DeviceID;               /* 设备ID */
    int ActionTaskID;           /* 算法任务ID */
    std::string ActionID;       /* 算法ID */
    std::string ActionName;     /* 算法名称 */
};



/**
 * @brief 摄像机线程需要的信息
 * 
 */
struct CameraThreadInfo
{
    std::string RedisIP;        /* Redis IP */
    int RedisPort;              /* Redis Port */
    std::string RedisPWD;       /* Redis Password */
    int DeviceID;               /* 设备ID */
    std::vector<std::string> vecAction;  /* Redis Key，一个设备可能会有多个算法ID */

    CameraThreadInfo() = default;
    CameraThreadInfo& operator=(const CameraThreadInfo& other) {
        if (this != &other) {
            RedisIP = other.RedisIP;
            RedisPort = other.RedisPort;
            RedisPWD = other.RedisPWD;
            DeviceID = other.DeviceID;
            vecAction = other.vecAction;
        }
        return *this;
    }
};


/**
 * @brief 一周某一天的时间
 * 
 */
struct WeekTime
{
    int WeekDay;                /* 星期几，0是按天计算的，可以直接无视 */
    QDateTime StartTime;        /* 开始时间 */
    QDateTime EndTime;          /* 结束时间 */
};


/**
 * @brief 功能检测时间段信息
 * 
 */
struct DetectPeriodInfo
{
    int ChannelID;                      /* 通道ID */
    AppFunction appFunction;            /* 应用功能 */
    std::string PeriodName;             /* 时段名称 */
    Enum_PeriodType PeriodType;         /* 时段类型 */
    QDateTime lastTime;                 /* 上次更新时间 */

    std::list<WeekTime> listWeekTime;   /* 一周的时间段 */

    DetectPeriodInfo();
    DetectPeriodInfo(const DetectPeriodInfo& other);
    DetectPeriodInfo& operator=(const DetectPeriodInfo& other);
};


/* ====================================================================================
 * *******************************    各种报警信息    **********************************
 * ====================================================================================*/


/**
 * @brief 人员信息
 * 
 */
struct PersonInfo
{
    std::string PersonID;       /* 人员ID，等于-1就是无法识别出身份的人员 */
    std::string PersonName;     /* 人员名称 */
    QDateTime lastOnWork;       /* 最后在岗时间 */
};

/**
 * @brief 报警信息
 * 
 */
struct AlarmInfo
{
    bool Is_Alarm;              /* 是否报警 */
    bool Is_InsertEQM;          /* 是否已经插入到EQM数据库中 */
    bool Is_OnWork;             /* 是否在岗 */
    int PKID;                   /* 主键ID，主要用来判断是否已经写入数据库 */
    int AlarmID;                /* 报警ID */
    int DeviceID;               /* 设备ID，数据库表格中对应的是CamerID */
    
    int RoomID;                 /* 房间ID */
    int ChannelID;              /* 通道ID */
    int State;                  /* 状态 */
    int OnWork;                 /* 是否在工作 */
    AppFunction appFunction;    /* 应用功能 */
    std::string ActionID;       /* 算法ID */
    QDateTime StartTime;        /* 报警开始时间 */
    QDateTime EndTime;          /* 报警结束时间 */
    QDateTime EventTime;        /* 事件时间（报警发生时间，在数据库里对应的是CreateTime） */
    // std::string PicUrl;         /* 报警图片URL */
    std::string ImageInfo;      /* 图片信息 */
    std::string ActionDes;      /* 算法描述信息 */
    std::string FaceIDList;     /* 人脸ID列表 */
    std::string FaceNameList;   /* 人脸名称列表 */
    std::list<std::string> listBbox;       /* Bbox列表，应该是图片中的报警位置 */
    std::list<PersonInfo> listPersonInfo;    /* 人员信息 */

    AlarmInfo();
    AlarmInfo(const AlarmInfo& other);
    AlarmInfo& operator=(AlarmInfo& other);

    void reInit();
    /* 获取人脸ID列表 */
    std::string getFaceIDListString();
    /* 获取人脸名称列表 */
    std::string getFaceNameListString();
};

/**
 * @brief 报警信息容器，并提供一些函数
 * 
 */
struct ListAlarmInfo
{
    ~ListAlarmInfo() {
        clearAlarmInfo();
    }
    std::list<AlarmInfo*> listAlarmInfo;    /* 报警信息列表 */

    /* 添加报警信息 */
    bool addAlarmInfo(AlarmInfo& info);
    /* 查找报警记录 */
    AlarmInfo* findAlarmInfo(AlarmInfo& info);
    /* 删除报警记录 */
    void deleteAlarmInfo(AlarmInfo& info);
    /* 清空报警记录 */
    void clearAlarmInfo();
};


/**
 * @brief 房间内的人脸信息
 * 
 */
struct RoomFaceInfo
{
    int ChannelID = 0;
    int RoomID = 0;
    int CameraID = 0;
    int MaxNum = 0;
    int MinNum = 0;
    QDateTime StartTime;
    QDateTime EndTime;
    std::list<PersonInfo> listPersonInfo;

    RoomFaceInfo() = default;
    RoomFaceInfo& operator=(const RoomFaceInfo& other) {
        if (this != &other) {
            ChannelID = other.ChannelID;
            RoomID = other.RoomID;
            CameraID = other.CameraID;
            MaxNum = other.MaxNum;
            MinNum = other.MinNum;
            StartTime = other.StartTime;
            EndTime = other.EndTime;
            listPersonInfo = other.listPersonInfo;
        }
        return *this;
    }
    /* 查找是否有相同的人脸信息 */
    bool findPersonInfo(const PersonInfo& info);
};

/**
 * @brief 房间内的人脸信息
 * 
 */
struct ListRoomFaceInfo
{
    std::list<RoomFaceInfo> listRoomFaceInfo;

    void addRoomFaceInfo(RoomFaceInfo& info);
    void addRoomFaceInfo(AlarmInfo& info);
    RoomFaceInfo* findRoomFaceInfo(RoomFaceInfo& info);
    RoomFaceInfo* findRoomFaceInfo(int ChannelID, int RoomID, int CameraID);
};





/**
 * @brief 房间和摄像机关联信息，这个是从客户端配置的
 *        一个房间对应多个摄像机
 * 
 */
struct RoomCameraInfo
{
    int RoomID;                     /* 房间ID */
    std::list<int> listCameraID;    /* 摄像机列表 */

    RoomCameraInfo() = default;
    RoomCameraInfo& operator=(const RoomCameraInfo& other) {
        if (this != &other) {
            RoomID = other.RoomID;
            listCameraID = other.listCameraID;
        }
        return *this;
    }
    /* 这里只对比RoomID */
    bool operator==(const RoomCameraInfo& other) const {
        if(RoomID != other.RoomID) {
            return false;
        }
        return true;
    }
};

/* ====================================================================================
 * ****************************    线程功能需要的结构体    *****************************
 * ====================================================================================*/



/**
 * @brief 单个算法信息，包括包含的摄像机，房间、频率信息
 * 
 */
struct ActionInfo
{
    RunTimeState RunState;          /* 线程运行状态 */
    int ChannelID;                  /* 通道ID */
    int RoomID;                     /* 房间ID */
    int CameraID;                   /* 摄像机ID */
    Enum_RoomType RoomType;         /* 房间类型 */
    std::string ActionID;           /* 算法ID */
    std::string strRoomName;        /* 房间名称 */
    std::string strActionName;      /* 算法名称，这个是自定义的，不需要对比 */

    ActionInfo();
    ActionInfo& operator=(const ActionInfo& other);
    /* 对比是否相等，这个需要完全相等，包括算法信息和摄像机信息 */
    bool operator==(const ActionInfo& other);
    /* 对比除摄像机外的基础信息是否相等 */
    bool isEqualBaseInfo(const ActionInfo& other);
    
};

/**
 * @brief 算法信息列表，这个列表既可以存储房间和算法ID的关联信息，也可以用来存储运行线程的信息
 * 
 */
class ListActionInfo
{
public:
    ListActionInfo() = default;

    /* 添加关联信息，会自动进行重复判断，如果已有相同的信息，则跳过 */
    bool insertActionInfo(ActionInfo* info);
    /* 删除信息 */
    bool deleteActionInfo(ActionInfo* info);
    /* 给算法添加摄像机，原有的会被替换掉 */
    bool addActionCamera(ActionInfo* pInfo);
    /* 清空算法中的摄像机信息 */
    void clearCameraList();
    /* 清空设置成STOP或ERROR的Action */
    void clearStopAction();
    /* 查找算法ID是否已在列表中 */
    ActionInfo* findActionInList(ActionInfo* pInfo);
    /* 查找算法ID是否在列表中，这里查找不会对比摄像机ID */
    ActionInfo* findActionIDInListNoCamera(ActionInfo* pInfo);
    
    /* 获取容器 */
    std::list<ActionInfo*>& getData() {
        return listActionInfo;
    }
    /* 清空容器 */
    void clear();

    std::list<ActionInfo*> listActionInfo;    /* 房间和算法ID关联列表 */

};

/**
 * @brief 算法信息，以房间分类，算法包含在这干房间内所需要的摄像机ID
 * 
 */
struct RoomActionInfo
{
    int ChannelID;                  /* 通道ID */
    int RoomID;                     /* 房间ID */
    Enum_RoomType RoomType;         /* 房间类型 */
    std::string ActionID;           /* 算法ID */
    std::string strRoomName;        /* 房间名称 */
    std::list<int> listCameraID;    /* 摄像机ID */

    RoomActionInfo() {
        ChannelID = -1;
        RoomID = -1;
        RoomType = Enum_RoomType::ROOM_NONE;
        ActionID = "";
        strRoomName = "";
        listCameraID.clear();
    }
    RoomActionInfo(const RoomActionInfo& other)
        : ChannelID(other.ChannelID), RoomID(other.RoomID), 
        RoomType(other.RoomType), ActionID(other.ActionID), 
        strRoomName(other.strRoomName), listCameraID(other.listCameraID) {}
    RoomActionInfo& operator=(const RoomActionInfo& other) {
        if (this != &other) {
            ChannelID = other.ChannelID;
            RoomID = other.RoomID;
            RoomType = other.RoomType;
            ActionID = other.ActionID;
            strRoomName = other.strRoomName;
            listCameraID = other.listCameraID;
        }
        return *this;
    }

    /* 对比频道信息、房间信息、算法ID是否相等 */
    bool isEqualBaseInfo(const RoomActionInfo& other);
    

};

/* 房间和算法ID关联列表 */
class ListRoomActionInfo
{
public:
    ListRoomActionInfo() = default;

    /* 添加关联信息，会自动进行重复判断，如果已有相同的room和action关联信息，则跳过 */
    bool insertRoomActionInfo(const RoomActionInfo& info);
    /* 添加关联信息，会自动进行重复判断，如果已有相同的room和action关联信息，则跳过 */
    bool insertRoomActionInfo(const int ChannelID, const int RoomID, const std::string& strActionID);
    /* 删除一个容器，注意，这个不能在别的for循环中删除，只能单独删除 */
    bool deleteRoomActionInfo(RoomActionInfo* pInfo);
    /* 清空容器 */
    void clear();
    /* 添加算法信息，根据传入的算法信息，自动将其加入到对应的容器中 */
    void addActionInfo(const ActionInfo& info);
    /* 清空算法对应的摄像机列表 */
    void clearCameraList();
    /* 清理设置为STOP或者ERROR的RoomAction */
    // void clearStopRoomAction();
    /* 查找算法ID是否已在列表中 */
    RoomActionInfo* findActionIDInList(const int chnID, const int RoomID, const std::string& strActionID);
    
    /* 获取容器 */
    std::list<RoomActionInfo*>& getData() {
        return listRoomActionInfo;
    }

    std::list<RoomActionInfo*> listRoomActionInfo;    /* 房间和算法ID关联列表 */

};

/**
 * @brief 房间内的摄像机和算法关联信息
 * 
 */
 struct RoomCamActInfo
 {
    int RoomID;                         /* 房间ID */
    Enum_RoomType RoomType;             /* 房间类型 */
    std::string strRoomName;            /* 房间名称 */
    std::map<int, std::list<std::string>> mapCameraAction;    /* 摄像机ID和算法ID关联信息，一个摄像机可以有多个算法列表 */

    RoomCamActInfo();
    RoomCamActInfo(const RoomCamActInfo& other);
    RoomCamActInfo& operator=(const RoomCamActInfo& other);
    /* 添加摄像机算法 */
    void addCameraAction(int CameraID, const std::string& ActionID);
    
 };


/**
 * @brief 通道和应用信息
 * 
 */
 struct ChannelAppInfo
 {
    uint32_t AppType;               /* 应用信息，按位计算 */
    int ChannelID;                  /* 通道ID */
    std::string strChannelName;     /* 通道名称 */
    std::string strAppName;         /* 应用名称 */

    ChannelAppInfo() = default;
    ChannelAppInfo& operator=(ChannelAppInfo& other) {
        if (this != &other) {
            AppType = other.AppType;
            ChannelID = other.ChannelID;
            strChannelName = other.strChannelName;
            strAppName = other.strAppName;
        }
        return *this;
    }
 };







#endif /* GLOBALVARIABLE_H */
